using System.Collections;
using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.InputSystem;


namespace BigSock {

	/*
		Interface that represents an ability.
	*/
	public interface IAbility {
		/*
			The cooldown of the ability.
		*/
		TimeSpan Cooldown { get; }

		/*
			The name of the ability.
		*/
		string Name { get; }
		
		/*
			The description of the ability.
		*/
		string Description { get; }

		/*
			The id of the ability.
		*/
		ulong Id { get; }

		/*
			The next time the ability has cooled down.
		*/
		DateTime NextTimeCanUse { get; }

		/*
			Whether the ability is ready.
		*/
		bool Ready { get; }


		/*
			-----------------------------
			Add in something for costs.
			- Uses
			- Mana
			- Stamina
			- HP
			-----------------------------
		*/



		/*
			Try to use the ability.
		*/
		bool Use(Character actor, Vector2 target);
	}
}