using System.Collections;
using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.InputSystem;
using BigSock.UI;


namespace BigSock {

	// Takes and handles input and movement for a player character
	public class PlayerController : Character
	{

			public HPBar hpBar;
			public float collisionOffset = 0.05f;
			public ContactFilter2D movementFilter;
			
			public GameObject attack;

			Vector2 movementInput;
			SpriteRenderer spriteRenderer;
			//Rigidbody2D rb;
			Animator animator;
			List<RaycastHit2D> castCollisions = new List<RaycastHit2D>();

			bool canMove = true;





			public DateTime NextTimeCanAttack { get; private set; } = DateTime.Now;




			// Start is called before the first frame update 
			void Start()
			{
					rb = GetComponent<Rigidbody2D>();
					animator = GetComponent<Animator>();
					spriteRenderer = GetComponent<SpriteRenderer>();
					hpBar.SetMaxHealth(Convert.ToInt32(MaxHP));
					hpBar.SetHealth(Convert.ToInt32(HP));
			}


			
			private void FixedUpdate() {
					if(canMove) {
							// If movement input is not 0, try to move
							if(movementInput != Vector2.zero){
									
									bool success = TryMove(movementInput);

									if(!success) {
											success = TryMove(new Vector2(movementInput.x, 0));
									}

									if(!success) {
											success = TryMove(new Vector2(0, movementInput.y));
									}
									
									animator.SetBool("isMoving", success);
							} else {
									animator.SetBool("isMoving", false);
							}

							// Set direction of sprite to movement direction
							var mouse = Camera.main.ScreenToWorldPoint(Input.mousePosition);
							var pos = transform.position;
							var temp = (mouse - pos);
							//temp.z = 0;
							//direction = temp.normalized;

							if(temp.x < 0) {
									spriteRenderer.flipX = true;
							} else if (temp.x > 0) {
									spriteRenderer.flipX = false;
							}
					}
					

			}

			private void Update()
			{
				if (Input.GetKeyDown(KeyCode.Space) || Input.GetMouseButton(0)) {
					// Manage attack cooldown.
					if(NextTimeCanAttack <= DateTime.Now) {
						NextTimeCanAttack = DateTime.Now.AddSeconds(AttackCooldown);

						Instantiate(attack, new Vector3(transform.position.x, transform.position.y, transform.position.z), attack.transform.rotation);
						//if(movementInput.x < 0 ) {
						//	Instantiate(attack, new Vector3(transform.position.x - 1.0f, transform.position.y, transform.position.z), attack.transform.rotation);
						//} else if(movementInput.x >= 0) {
						//	Instantiate(attack, new Vector3(transform.position.x + 1.0f, transform.position.y, transform.position.z), attack.transform.rotation);
						//}

					}

					
				}
			}


			

			private bool TryMove_OLD(Vector2 direction) {
					if(direction != Vector2.zero) {
							// Check for potential collisions
							int count = rb.Cast(
									direction, // X and Y values between -1 and 1 that represent the direction from the body to look for collisions
									movementFilter, // The settings that determine where a collision can occur on such as layers to collide with
									castCollisions, // List of collisions to store the found collisions into after the Cast is finished
									(float) MovementSpeed * Time.fixedDeltaTime + collisionOffset); // The amount to cast equal to the movement plus an offset

							if(count == 0){
									rb.MovePosition(rb.position + direction * (float) MovementSpeed * Time.fixedDeltaTime);
									return true;
							} else {
									return false;
							}
					} else {
							// Can't move if there's no direction to move in
							return false;
					}
			
					// 
			}

			void OnMove(InputValue movementValue) {
					movementInput = movementValue.Get<Vector2>();
			}


			public void LockMovement() {
					canMove = false;
			}

			public void UnlockMovement() {
					canMove = true;
			}

			/*
			Method for what to do when the character takes damage.
			*/
			protected override void OnDamage(IAttackStats attack)  {
				base.OnDamage(attack);
				hpBar.SetHealth(Convert.ToInt32(HP));
			}
	}
}
