using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Linq;
using System;


namespace BigSock {

	public partial class EnemyController : Character {
		public float collisionOffset = 0.05f;
		public ContactFilter2D movementFilter;
		List<RaycastHit2D> castCollisions = new List<RaycastHit2D>();

		private Transform target;
		Animator m_Animator;
		private float distance; 

		private float canAttack;

		private EmptyCollider followCollider;
		private EmptyCollider attackCollider;

		private bool isInMelee = false;


		//Rigidbody2D rb;

		// private Collider2D_Proxy secondCollider;
		
		void Start(){
			rb = GetComponent<Rigidbody2D>();

			m_Animator = gameObject.GetComponent<Animator>();

			followCollider = transform.Find("followCollider").GetComponent<EmptyCollider>();
			followCollider.OnColliderEnter2D_Action += Move_OnColliderEnter2D;
			followCollider.OnColliderStay2D_Action += Move_OnColliderStay2D;
			followCollider.OnColliderExit2D_Action += Move_OnColliderExit2D;

			attackCollider = transform.Find("MeleeCollider").GetComponent<EmptyCollider>();
			attackCollider.OnColliderEnter2D_Action += Attack_OnColliderEnter2D;
			attackCollider.OnColliderStay2D_Action += Attack_OnColliderStay2D;
			attackCollider.OnColliderExit2D_Action += Attack_OnColliderExit2D;
		}







		private void RotateAnimation(Vector2 direction) {
			if (direction.x > 0.01f){
				gameObject.GetComponent<SpriteRenderer>().flipX = false;
			}
			else if (direction.x < -0.01f){
				gameObject.GetComponent<SpriteRenderer>().flipX = true;
			}
		}

		private void Update(){
			if (target != null && !isInMelee){

				/* //walk
				float step = speed * Time.deltaTime;
				transform.position = Vector2.MoveTowards(transform.position, target.position, step);
				
				//distance = Vector3.Distance (transform.position, target.position);
				//roter
				RotateAnimation();
				*/
				
				TryMove((new Vector2(target.position.x, target.position.y) - rb.position).normalized);
			}
			else{

			}
		}

		

		private bool TryMove_OLD(Vector2 direction) {
	
			if(direction != Vector2.zero) {
			
				// Check for potential collisions
				int count = rb.Cast(
						direction, // X and Y values between -1 and 1 that represent the direction from the body to look for collisions
						movementFilter, // The settings that determine where a collision can occur on such as layers to collide with
						castCollisions, // List of collisions to store the found collisions into after the Cast is finished
						(float) MovementSpeed * Time.fixedDeltaTime + collisionOffset); // The amount to cast equal to the movement plus an offset
				//Debug.Log($"cast {string.Join(", ", castCollisions.Select(x => $"{x.collider.isTrigger}"))}");
				//Debug.Log($"rb.position : {rb.position}");
				//Debug.Log($"target.position : {target.position}");
				//Debug.Log($"direction : {direction}");
				if(count == 0){
					rb.MovePosition(rb.position + direction * (float) MovementSpeed * Time.fixedDeltaTime);
					//print($"rb.position {rb.position}");
					//print($"direction {direction}");
					RotateAnimation(direction);

					return true;
				} else {
					return false;
				}
			} else {
				// Can't move if there's no direction to move in
				return false;
			}
			
		}
	}


	/*
		Attack
	*/
	public partial class EnemyController { 

		private void Attack_OnColliderEnter2D(Collider2D other){
			if (other.gameObject.tag == "Player")
				isInMelee = true;
		}

		

		private void Attack_OnColliderStay2D(Collider2D other) {
			var player = other.gameObject.GetComponent<PlayerController>();
			if(player != null) {
				// Create attack object.
				var attack = new AttackStats{
					Damage = Damage,
					Knockback = KnockbackForce,
					Range = 0,
					AttackSpeed = AttackSpeed,
					Source = transform.position,
				};

				// Get the player to take the damage.
				if(player.TakeDamage(attack)){
					//rb position - player position
					//Vector2 difference = (other.transform.position - transform.position).normalized;
					//player.KnockBack(KnockbackForce, difference);
					//animer nå ?
				}
			} 
		}
		private void Attack_OnColliderExit2D(Collider2D other){
				if (other.gameObject.tag == "Player")
						isInMelee = false;
		}

	}

	/*
		Move
	*/
	public partial class EnemyController { 

		private void Move_OnColliderEnter2D(Collider2D other)
		{
				Debug.Log("enter");
				if (other.gameObject.tag == "Player"){
							Debug.Log("enter if");
				
						m_Animator.SetTrigger("walk");
						target = other.transform;
				}
		}

		private void Move_OnColliderStay2D(Collider2D other)
		{

				if (other.gameObject.tag == "Player"){
				
		} 
		}

		private void Move_OnColliderExit2D(Collider2D other)
		{
				if (other.gameObject.tag == "Player"){
						m_Animator.SetTrigger("idle");
						target = other.transform;
						target = null;        
				}      
		}
			
	}

}
